@@ -2,6 +2,7 @@ import datetime
 import operator
 
 from flask import request
+from flask_peewee.utils import models_to_path
 from peewee import *
 from wtforms import fields, form, widgets
 from wtfpeewee.fields import ModelSelectField, ModelSelectMultipleField
@@ -68,49 +69,138 @@ FIELDS_TO_LOOKUPS = {
 CONVERTERS = {
     (ForeignKeyField, 'eq'): lambda f: ModelSelectField(model=f.to),
     (ForeignKeyField, 'in'): lambda f: ModelSelectMultipleField(model=f.to),
-    (PrimaryKeyField, 'eq'): lambda f: ModelSelectField(model=f.model),
-    (PrimaryKeyField, 'in'): lambda f: ModelSelectMultipleField(model=f.model),
     (DateTimeField, 'today'): lambda f: fields.HiddenField(),
     (DateTimeField, 'yesterday'): lambda f: fields.HiddenField(),
     (DateTimeField, 'this_week'): lambda f: fields.HiddenField(),
     (BooleanField, 'eq'): lambda f: BooleanSelectField(),
 }
 
+
+def is_valid_lookup(model, lookup):
+    lookups = lookup.split('__')
+    lookup_type = 'eq'
+    
+    if len(lookups) > 1 and lookups[-1] in LOOKUP_TYPES:
+        lookup_type = lookups.pop()
+    
+    curr = model
+    for rel_field in lookups[:-1]:
+        if rel_field in curr._meta.rel_fields:
+            # is this a lookup using the descriptor name of a foreign key field,
+            # if so change it to the actual field name, e.g. user -> user_id
+            rel_field = curr._meta.rel_fields[rel_field]
+        
+        if rel_field in curr._meta.fields:
+            field_obj = curr._meta.fields[rel_field]
+            if isinstance(field_obj, ForeignKeyField):
+                curr = field_obj.to
+            else:
+                return False
+        else:
+            if rel_field in curr._meta.reverse_relations:
+                curr = curr._meta.reverse_relations[rel_field]
+            else:
+                return False
+    
+    lookup = lookups[-1]
+    if lookup in curr._meta.rel_fields:
+        lookup = curr._meta.rel_fields[lookup]
+
+    if lookup not in curr._meta.fields:
+        return False
+    
+    field_obj = curr._meta.fields[lookup]
+    if lookup_type not in FIELDS_TO_LOOKUPS[INV_FIELD_TYPES[type(field_obj)]]:
+        return False
+    
+    return (curr, lookup, lookup_type)
+
 def lookups_for_field(field):
+    """
+    Get a list of valid lookups for a given field type.  Lookups are expressed
+    as a 2-tuple of (<lookup suffix>, <human description>)
+    
+    >>> lookups_for_field(User.username)
+    [('eq', 'equal to'), ('icontains', 'contains'), ('istartswith', 'starts with')]
+    """
     field_class = type(field)
     return [
         (lookup, LOOKUP_TYPES[lookup]) \
             for lookup in FIELDS_TO_LOOKUPS[INV_FIELD_TYPES[field_class]]
     ]
 
-def form_field_for_lookup(field, lookup):
-    field_class = type(field)
-    default = lambda f: fields.TextField()
-    return CONVERTERS.get((field_class, lookup), default)(field)
+class Lookup(object):
+    def __init__(self, field):
+        self.field = field
+        
+        self.field_class = type(self.field)
+        self.field_name = self.field.name
+        self.verbose_name = self.field.verbose_name
+        self.model = field.model
+    
+    def __repr__(self):
+        return '<Lookups for: %s.%s>' % (self.model.__name__, self.field_name)
 
-def get_fields(model):
-    fields = {}
-    for field_name, field_obj in model._meta.fields.items():
-        field_class = type(field_obj)
-        for lookup in FIELDS_TO_LOOKUPS[INV_FIELD_TYPES[field_class]]:
-            form_field = form_field_for_lookup(field_obj, lookup)
-            if form_field:
-                fields['%s__%s' % (field_name, lookup)] = form_field
-    return fields
+    def __eq__(self, rhs):
+        return self.field == rhs.field
 
-def get_filter_form(model):
-    frm = form.BaseForm(get_fields(model))
-    frm.process(None)
-    return frm
+    def get_field_type(self):
+        return INV_FIELD_TYPES[self.field_class]
+    
+    def get_lookups(self):
+        return lookups_for_field(self.field)
+    
+    def to_context(self):
+        return dict(
+            name=self.field_name,
+            verbose_name=self.verbose_name,
+            field_type=self.get_field_type(),
+            lookups=self.get_lookups(),
+            prefix=prefix,
+        )
+    
+    def html_fields(self):
+        default = lambda f: fields.TextField()
+        return dict([
+            (
+                '%s__%s' % (self.field_name, lookup[0]),
+                CONVERTERS.get((self.field_class, lookup[0]), default)(self.field)
+            ) for lookup in self.get_lookups()
+        ])
 
-def get_lookups(model, exclude):
-    return [
-        (f, lookups_for_field(f)) for f in model._meta.get_fields() if f not in exclude
-    ]
+class ModelLookup(object):
+    def __init__(self, model, exclude, path, raw_id_fields):
+        self.model = model
+        self.exclude = exclude
+        self.path = path
+        self.raw_id_fields = raw_id_fields
+    
+    def get_lookups(self):
+        return [
+            Lookup(f) for f in self.model._meta.get_fields() \
+                if f.name not in self.exclude
+        ]
+    
+    def get_prefix(self):
+        if not self.path:
+            return ''
+        return models_to_path(self.path) + '__'
+    
+    def get_html_fields(self):
+        fields = {}
+        for lookup in self.get_lookups():
+            fields.update(lookup.html_fields())
+        return fields
+    
+    def html_form(self):
+        frm = form.BaseForm(self.get_html_fields(), prefix=self.get_prefix())
+        frm.process(None)
+        return frm
 
 def _rd(n):
     return datetime.date.today() + datetime.timedelta(days=n)
 
+
 class FilterPreprocessor(object):
     def process_lookup(self, raw_lookup, values=None):
         """
@@ -156,18 +246,33 @@ class FilterPreprocessor(object):
 
 
 class QueryFilter(object):
-    def __init__(self, query, exclude_lookups=None, ignore_filters=None):
+    def __init__(self, query, exclude_fields=None, ignore_filters=None, raw_id_fields=None, related=None):
         self.query = query
         self.model = self.query.model
 
-        self.exclude_lookups = exclude_lookups or ()
+        # fix any foreign key fields
+        self.exclude_fields = []
+        for field_name in exclude_fields or ():
+            if field_name in self.model._meta.fields:
+                self.exclude_fields.append(field_name)
+            else:
+                self.exclude_fields.append(self.model._meta.rel_fields[field_name])
+        
         self.ignore_filters = ignore_filters or ()
+        self.raw_id_fields = raw_id_fields or ()
+        
+        # a list of related QueryFilter() objects
+        self.related = related or ()
     
-    def get_form(self):
-        return get_filter_form(self.model)
-    
-    def get_lookups(self):
-        return get_lookups(self.model, self.exclude_lookups)
+    def get_model_lookups(self, path=None):
+        model_lookups = [ModelLookup(self.model, self.exclude_fields, path, self.raw_id_fields)]
+        path = path or []
+        path.append(self.model)
+        for related_filter in self.related:
+            model_lookups.extend(
+                related_filter.get_model_lookups(list(path))
+            )
+        return model_lookups
     
     def get_preprocessor(self):
         return FilterPreprocessor()
@@ -186,9 +291,16 @@ class QueryFilter(object):
         for key in request.args:
             if key in self.ignore_filters:
                 continue
+
+            if not is_valid_lookup(self.model, key):
+                continue
             
             values = request.args.getlist(key)
-            self.raw_lookups.append((key, values))
+            if key.endswith('__in'):
+                self.raw_lookups.append((key, values))
+            else:
+                for v in values:
+                    self.raw_lookups.append((key, v))
             
             # preprocessor returns a list of filters to apply
             filters.extend(preprocessor.process_lookup(key, values))
