@@ -1,16 +1,19 @@
+from __future__ import with_statement
+
 import datetime
 
-from flask import request, session, url_for
+from flask import request, session, url_for, g
 
 from flask_peewee.admin import ModelAdmin, AdminPanel
+from flask_peewee.filters import Lookup
 from flask_peewee.tests.base import FlaskPeeweeTestCase
-from flask_peewee.tests.test_app import User, Message, Note, admin
+from flask_peewee.tests.test_app import User, Message, Note, admin, AModel, BModel, CModel, DModel, BDetails
 from flask_peewee.utils import get_next, make_password, check_password
 
 from wtfpeewee.orm import model_form
 
 
-class AdminTestCase(FlaskPeeweeTestCase):
+class BaseAdminTestCase(FlaskPeeweeTestCase):
     def login(self, context=None):
         context = context or self.app
         context.post('/accounts/login/', data={
@@ -21,7 +24,9 @@ class AdminTestCase(FlaskPeeweeTestCase):
     def logout(self, context=None):
         context = context or self.app
         context.post('/accounts/logout/')
-    
+
+
+class AdminTestCase(BaseAdminTestCase):
     def test_admin_auth(self):
         self.create_users()
         
@@ -81,6 +86,11 @@ class AdminTestCase(FlaskPeeweeTestCase):
         # check that we have the stuff from the auth module and the index view
         self.assertContext('user', self.admin)
         self.assertContext('model_admins', [
+            admin._registry['amodel'],
+            admin._registry['bdetails'],
+            admin._registry['bmodel'],
+            admin._registry['cmodel'],
+            admin._registry['dmodel'],
             admin._registry['message'],
             admin._registry['note'],
             admin._registry['user'],
@@ -381,8 +391,7 @@ class AdminTestCase(FlaskPeeweeTestCase):
                 self.admin,
             ])
             
-            # test a lookup using "in"
-            resp = c.get('/admin/user/?username__in=admin&username__in=normal&ordering=-username')
+            resp = c.get('/admin/user/?username__eq=admin&username__eq=normal&ordering=-username')
             self.assertEqual(resp.status_code, 200)
 
             query = self.get_context('query')
@@ -392,7 +401,7 @@ class AdminTestCase(FlaskPeeweeTestCase):
             ])
             
             # test a lookup using partial string
-            resp = c.get('/admin/user/?username__startswith=norm&ordering=-username')
+            resp = c.get('/admin/user/?username__istartswith=norm&ordering=-username')
             self.assertEqual(resp.status_code, 200)
 
             query = self.get_context('query')
@@ -473,6 +482,121 @@ class AdminTestCase(FlaskPeeweeTestCase):
             self.assertEqual(note.message, 'testing')
 
 
+class AdminFilterTestCase(BaseAdminTestCase):
+    def setUp(self):
+        super(AdminFilterTestCase, self).setUp()
+        BDetails.drop_table(True)
+        DModel.drop_table(True)
+        CModel.drop_table(True)
+        BModel.drop_table(True)
+        AModel.drop_table(True)
+        AModel.create_table()
+        BModel.create_table()
+        CModel.create_table()
+        DModel.create_table()
+        BDetails.create_table()
+    
+    def create_models(self):
+        for i in range(1, 4):
+            a = AModel.create(a_field='a%d' % i)
+            b = BModel.create(b_field='b%d' % i, a=a)
+            c = CModel.create(c_field='c%d' % i, b=b)
+            d = DModel.create(d_field='d%d' % i, c=c)
+            if i % 2 == 0:
+                bd = BDetails.create(b=b)
+    
+    def test_filters(self):
+        users = self.create_users()
+        self.create_models()
+        
+        with self.flask_app.test_client() as c:
+            self.login(c)
+
+            resp = c.get('/admin/dmodel/?c__b__a__a_field=a1')
+            query = self.get_context('query')
+            
+            self.assertEqual([o.d_field for o in query.get_list()], ['d1'])
+            
+            resp = c.get('/admin/dmodel/?c__b__a__a_field=a3')
+            query = self.get_context('query')
+            
+            self.assertEqual([o.d_field for o in query.get_list()], ['d3'])
+            
+            resp = c.get('/admin/dmodel/?c__b__a=2')
+            query = self.get_context('query')
+            
+            self.assertEqual([o.d_field for o in query.get_list()], ['d2'])
+    
+    def assertLookups(self, lookups, expected):
+        """
+        Pass in something like (field_name, prefix, models)
+        """
+        flattened = []
+        for model_lookup in lookups:
+            flattened.extend(model_lookup.get_lookups())
+        self.assertEqual(flattened, expected)
+    
+    def test_lookups(self):
+        users = self.create_users()
+        
+        with self.flask_app.test_client() as c:
+            self.login(c)
+
+            resp = c.get('/admin/amodel/')
+            query_filter = self.get_context('query_filter')
+            model_lookups = query_filter.get_model_lookups()
+            self.assertLookups(model_lookups, [
+                Lookup(AModel.id),
+                Lookup(AModel.a_field),
+            ])
+            
+            resp = c.get('/admin/bmodel/')
+            query_filter = self.get_context('query_filter')
+            lookups = query_filter.get_model_lookups()
+            self.assertLookups(lookups, [
+                Lookup(BModel.id),
+                Lookup(BModel.a),
+                Lookup(BModel.b_field),
+                Lookup(AModel.id),
+                Lookup(AModel.a_field),
+            ])
+            
+            resp = c.get('/admin/cmodel/')
+            query_filter = self.get_context('query_filter')
+            lookups = query_filter.get_model_lookups()
+            self.assertLookups(lookups, [
+                Lookup(CModel.id),
+                Lookup(CModel.b),
+                Lookup(CModel.c_field),
+                Lookup(BModel.id),
+                Lookup(BModel.a),
+                Lookup(BModel.b_field),
+                Lookup(AModel.id),
+                Lookup(AModel.a_field),
+                Lookup(BDetails.id),
+                Lookup(BDetails.b),
+            ])
+            
+            resp = c.get('/admin/dmodel/')
+            query_filter = self.get_context('query_filter')
+            lookups = query_filter.get_model_lookups()
+            self.assertLookups(lookups, [
+                Lookup(DModel.id),
+                Lookup(DModel.c),
+                Lookup(DModel.d_field),
+                Lookup(CModel.id),
+                Lookup(CModel.b),
+                Lookup(CModel.c_field),
+                Lookup(BModel.id),
+                Lookup(BModel.a),
+                Lookup(BModel.b_field),
+                Lookup(AModel.id),
+                Lookup(AModel.a_field),
+                Lookup(BDetails.id),
+                Lookup(BDetails.b),
+            ])
+
+
 class TemplateHelperTestCase(FlaskPeeweeTestCase):
     def setUp(self):
         super(TemplateHelperTestCase, self).setUp()
@@ -513,6 +637,11 @@ class TemplateHelperTestCase(FlaskPeeweeTestCase):
     
     def test_get_model_admins(self):
         self.assertEqual(self.th.get_model_admins(), {'model_admins': [
+            admin._registry['amodel'],
+            admin._registry['bdetails'],
+            admin._registry['bmodel'],
+            admin._registry['cmodel'],
+            admin._registry['dmodel'],
             admin._registry['message'],
             admin._registry['note'],
             admin._registry['user'],
